/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#ifndef AST_h
#define AST_h 1

#include "config.h"
#include "parser.h"
#include "structures.h"
#include "sym_table.h"

typedef unsigned Label;

struct Expr_Ident {
    const char	       *name;
    unsigned		slot;
};

struct Expr_Binary {
    Expr       	       *lhs, *rhs;
};

enum Arg_Kind {
    ARG_NORMAL, ARG_SPLICE
};

struct Arg_List {
    Arg_List	       *next;
    enum Arg_Kind	kind;
    Expr	       *expr;
};

struct Expr_Call {
/*    enum Function	func;*/
    unsigned           func;
    Arg_List	       *args;
};

struct Expr_Verb {
    Expr	       *obj, *verb;
    Arg_List	       *args;
};

struct Expr_Range {
    Expr	       *base, *from, *to;
};

struct Expr_Cond {
    Expr	       *condition, *consequent, *alternate;
};

enum Expr_Kind {
    EXPR_VAR, EXPR_ID,
    EXPR_PROP, EXPR_VERB,
    EXPR_INDEX, EXPR_RANGE,
    EXPR_ASGN, EXPR_CALL,
    EXPR_PLUS, EXPR_MINUS, EXPR_TIMES, EXPR_DIVIDE, EXPR_MOD, EXPR_NEGATE,
    EXPR_AND, EXPR_OR, EXPR_NOT,
    EXPR_EQ, EXPR_NE, EXPR_LT, EXPR_LE, EXPR_GT, EXPR_GE,
    EXPR_IN, EXPR_LIST, EXPR_COND,
    SizeOf_Expr_Kind /* The last element is also the number of elements... */
};

union Expr_Data {
    Var			var;
    struct Expr_Ident	id;
    struct Expr_Binary	bin;
    struct Expr_Call	call;
    struct Expr_Verb	verb;
    struct Expr_Range	range;
    struct Expr_Cond	cond;
    Expr	       *expr;
    Arg_List	       *list;
};

struct Expr {
    enum Expr_Kind	kind;
    union Expr_Data	e;
};

struct Stmt_Cond {
    struct Cond_Arm    *arms;
    Stmt	       *otherwise;
};

struct Cond_Arm {
    struct Cond_Arm    *next;
    Expr	       *condition;
    Stmt	       *stmt;
};

struct Stmt_List {
    struct Expr_Ident	id;
    Expr	       *expr;
    Stmt	       *body;
};

struct Stmt_Range {
    struct Expr_Ident	id;
    Expr	       *from, *to;
    Stmt	       *body;
};

struct Stmt_Loop {
    Expr	       *condition;
    Stmt	       *body;
};

struct Stmt_Fork {
    struct Expr_Ident	id;
    Expr	       *time;
    Stmt	       *body;
};

enum Stmt_Kind {
    STMT_COND, STMT_LIST, STMT_RANGE, STMT_WHILE, STMT_FORK, STMT_EXPR,
    STMT_RETURN
};

union Stmt_Data {
    struct Stmt_Cond	cond;
    struct Stmt_List	list;
    struct Stmt_Range	range;
    struct Stmt_Loop	loop;
    struct Stmt_Fork	fork;
    Expr	       *expr;
};

struct Stmt {
    Stmt	       *next;
    enum Stmt_Kind	kind;
    union Stmt_Data	s;
};
    

extern void	begin_code_allocation(void);
extern void	end_code_allocation(int);

extern Stmt	*alloc_stmt(enum Stmt_Kind);
extern Cond_Arm	*alloc_cond_arm(Expr *, Stmt *);
extern Expr	*alloc_expr(enum Expr_Kind);
extern Expr	*alloc_var(var_type);
extern Expr	*alloc_binary(enum Expr_Kind, Expr *, Expr *);
extern Expr	*alloc_verb(Expr *, Expr *, Arg_List *);
extern Arg_List	*alloc_arg_list(enum Arg_Kind, Expr *);
extern const char *ak_alloc_string(const char *);
extern void	ak_dealloc_string(const char *);

extern void	deallocate(void *);

extern Program *new_program_copying_stmt(Stmt *, unsigned, Names **);

/*extern void	add_ref_to_program(Program *); */
extern void	free_program(Program *);
extern void     free_stmt(Stmt *);

#endif /* !AST_h */

/* $Log: ast.h,v $
 * Revision 1.8  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.7  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.6  1992/08/31  22:21:43  pjames
 * Changed some `char *'s to `const char *'
 *
 * Revision 1.5  1992/08/28  23:13:36  pjames
 * Added ASGN_RANGE to assignment type enumeration.
 * Added range arm to assigment union.
 *
 * Revision 1.4  1992/08/14  00:15:28  pavel
 * Removed trailing comma in an enumeration.
 *
 * Revision 1.3  1992/08/14  00:01:11  pavel
 * Converted to a typedef of `var_type' = `enum var_type'.
 *
 * Revision 1.2  1992/07/30  21:21:21  pjames
 * Removed max_stack from AST structures.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
